/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useState, useEffect } from 'react';
import { TextField, Button, NativeSelect } from '@mui/material';
import styles from './add-database-form.module.scss';
import { AddDatabaseRequest, DatabaseType, databaseNameMapping, databaseProtocolMapping } from '@graphpolaris/shared/lib/data-access';

/** AddDatabaseFormProps is an interface containing the AuthViewModel. */
export interface AddDatabaseFormProps {
  styles: any;
  open: boolean;
  onClose(): void;
  onSubmit(data: AddDatabaseRequest): void;
}

/** AddDatabaseFormState is an interface containing the databasehost information. */
export interface AddDatabaseFormState extends AddDatabaseRequest {
  // username: string;
  // password: string;
  // hostname: string;
  // port: number;
  // databaseName: string;
  // internalDatabase: string;
  // databaseType: string;
  // styles: Record<string, string>; // TODO: check if needed
}

/** AddDatabaseForm is the View implementation for the connect screen that will be rendered. */
export default function AddDatabaseForm(props: AddDatabaseFormProps) {
  const [state, setState] = useState<AddDatabaseFormState>({
    // username: 'root',
    // password: 'DikkeDraak',
    // url: 'https://datastrophe.science.uu.nl/',
    // port: 8529,
    // name: 'Tweede Kamer Dataset',
    // internal_database_name: 'TweedeKamer',
    // type: DatabaseType.ArangoDB,

    // username: 'neo4j',
    // password: 'oL3nNlebrx4le2A0zxaFVqAo3HAvodHxwEiI_7_2JxI',
    // url: '635176c8.databases.neo4j.io',
    // port: 7687,
    // name: 'neo4j',
    // internal_database_name: 'neo4j',
    // type: DatabaseType.Neo4j,

    username: 'neo4j',
    password: 'StrongPass2022',
    url: 'localhost',
    port: 7687,
    name: 'neo4j',
    protocol: 'neo4j://',
    internal_database_name: 'neo4j',
    type: DatabaseType.Neo4j,
  });
  const [portValidation, setPortValidation] = useState<string>('valid');

  useEffect(() => {
    if (state && state.port && state.port > 9999) {
      setPortValidation('error');
    } else {
      setPortValidation('valid');
    }
  }, [state.port]);

  useEffect(() => {
    if (!state) return;
    if (state.type === DatabaseType.ArangoDB && state.port === 7687) {
      setState({ ...state, port: 8529 });
    } else if (state.type === DatabaseType.Neo4j && state.port === 8529) {
      setState({ ...state, port: 7687 });
    }
  }, [state.type]);

  /**
   * Validates if the port value is numerical. Only then will the state be updated.
   * @param port The new port value.
   */
  function handlePortChanged(port: string): void {
    if (!isNaN(Number(port))) setState({ ...state, port: Number(port) });
  }

  /** Handles the submit button click. Calls the onSubmit in the props with all the fields. */
  function handleSubmitClicked(): void {
    props.onSubmit(state);
  }

  return props.open ? (
    // return (
    <div
      className={styles.wrapper}
      onMouseDown={() => {
        props.onClose();
      }}
    >
      <div
        className={styles.authWrapper}
        onMouseDown={(e) => {
          e.stopPropagation();
        }}
      >
        <h1 className={styles.header}>Database Connect</h1>
        <form
          className={styles.formWrapper}
          onSubmit={(event: React.FormEvent) => {
            event.preventDefault();
            handleSubmitClicked();
          }}
        >
          <div className={styles.loginContainer}>
            <TextField
              className={styles.passLabel}
              label="Database name"
              type="databaseName"
              value={state.name}
              onChange={(event) =>
                setState({
                  ...state,
                  name: event.currentTarget.value,
                })
              }
              required
            />
          </div>
          <div className={styles.loginContainer}>
            <NativeSelect
              className={styles.passLabel}
              value={databaseNameMapping[state.type]}
              onChange={(event) => {
                setState({
                  ...state,
                  type: databaseNameMapping.indexOf(event.currentTarget.value),
                });
              }}
            >
              {databaseNameMapping.map((dbName) => (
                <option value={dbName} key={dbName}>
                  {dbName}
                </option>
              ))}
            </NativeSelect>
          </div>
          <div className={styles.loginContainer}>
            <NativeSelect
              className={styles.passLabel}
              value={state.protocol}
              onChange={(event) => {
                setState({
                  ...state,
                  protocol: event.currentTarget.value,
                });
              }}
            >
              {databaseProtocolMapping.map((protocol) => (
                <option value={protocol} key={protocol}>
                  {protocol}
                </option>
              ))}
            </NativeSelect>
          </div>
          <div className={styles.loginContainerRow}>
            <TextField
              className={styles.hostLabel}
              label="Hostname/IP"
              type="hostname"
              value={state.url}
              onChange={(event) =>
                setState({
                  ...state,
                  url: event.currentTarget.value,
                })
              }
              required
            />
            {portValidation && portValidation == 'error' ? (
              <TextField
                error
                className={styles.portLabel}
                label="Port error"
                type="port"
                value={state.port}
                onChange={(event) => handlePortChanged(event.currentTarget.value)}
                required
              />
            ) : (
              <TextField
                className={styles.portLabel}
                label="Port"
                type="port"
                value={state.port}
                onChange={(event) => handlePortChanged(event.currentTarget.value)}
                required
              />
            )}
          </div>
          <div className={styles.loginContainer}>
            <TextField
              className={styles.userLabel}
              label="Username"
              type="username"
              value={state.username}
              onChange={(event) =>
                setState({
                  ...state,
                  username: event.currentTarget.value,
                })
              }
              required
            />
          </div>
          <div className={styles.loginContainer}>
            <TextField
              className={styles.passLabel}
              label="Password"
              type="password"
              value={state.password}
              onChange={(event) =>
                setState({
                  ...state,
                  password: event.currentTarget.value,
                })
              }
              required
            />
          </div>
          <div className={styles.loginContainer}>
            <TextField
              className={styles.passLabel}
              label="Internal database"
              type="internalDatabaseName"
              value={state.internal_database_name}
              onChange={(event) =>
                setState({
                  ...state,
                  internal_database_name: event.currentTarget.value,
                })
              }
              required
            />
          </div>
          <div className={styles.loginContainerButton}>
            {portValidation && portValidation == 'error' ? (
              <Button disabled variant="contained" type="submit" color="success">
                Submit
              </Button>
            ) : (
              <Button variant="contained" type="submit" color="success">
                Submit
              </Button>
            )}

            <Button
              className={styles.cancelButton}
              variant="outlined"
              color="error"
              onClick={() => {
                props.onClose();
              }}
            >
              Cancel
            </Button>
          </div>
        </form>
      </div>
    </div>
  ) : (
    //);
    <></>
  );
}
