/**
 * This program has been developed by a student from the bachelor Information Science at
 * Utrecht University within the Research Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import { CompositeLayer } from 'deck.gl/typed';
import { ScatterplotLayer, LineLayer } from '@deck.gl/layers/typed';
import { LayerProps } from '../types';
import { getColor } from '../utils';
import { AggregateCommand } from '../aggregateCommand';
import { BrushingExtension } from '@deck.gl/extensions/typed';

export class FacetingLayer extends CompositeLayer<LayerProps> {
  /**
   * Faceting layer implements attribute driven faceting
   * Main drawback is the predefined attributes (which not every data set has)
   * Also, the data has to be duplicated in order to be able to go back to initial view
   */
  static componentName = 'FacetingLayer Layer';

  shouldUpdateState({ changeFlags }: any) {
    return changeFlags.somethingChanged;
  }

  updateState({ props, oldProps, changeFlags }: any) {
    const rebuildIndex = changeFlags.dataChanged;
    const zoom = Math.floor(this.context.viewport.zoom);

    if (rebuildIndex || zoom !== this.state.zoom) {
      // Aggregate based on predefined levels
      if (zoom > 5) props.graph.executeCommand(new AggregateCommand('attributes.city'));
      if (zoom <= 5 && zoom > 3) props.graph.executeCommand(new AggregateCommand('attributes.region'));
      if (zoom < 3 && zoom > 2) props.graph.executeCommand(new AggregateCommand('attributes.country'));
      if (zoom < 2) props.graph.executeCommand(new AggregateCommand('attributes.continent'));

      this.setState({
        nodes: props.graph.data.nodes,
        edges: props.graph.data.edges,
        zoom: zoom,
      });
    }
  }

  renderLayers() {
    return [
      new LineLayer({
        id: 'edges',
        data: this.state.edges,
        pickable: true,
        extensions: [new BrushingExtension()],
        brushingEnabled: this.props.brushing,
        brushingRadius: 500000,
        brushingTarget: 'source_target',
        getWidth: (d) => d.count,
        getSourcePosition: (d) => d.from_coordinates,
        getTargetPosition: (d) => d.to_coordinates,
        getColor: (d) => getColor(d.attributes.prediction, this.props.colorScale),
      }),
      new ScatterplotLayer({
        id: 'nodes',
        data: this.state.nodes,
        pickable: true,
        opacity: 0.6,
        interactive: true,
        filled: true,
        radiusScale: 1,
        radiusMinPixels: 4,
        radiusMaxPixels: 20,
        getPosition: (d) => d.attributes.coordinates,
        getFillColor: [0, 0, 0],
      }),
    ];
  }
}
